﻿namespace Nop.Plugin.Payments.UsaEpay.Tests
{
    #region

    using System.Linq;

    using Microsoft.VisualStudio.TestTools.UnitTesting;

    using Nop.Core.Domain.Payments;
    using Nop.Services.Payments;

    #endregion

    /// <summary>
    ///     The transaction response processor tests.
    /// </summary>
    [TestClass]
    public class TransactionResponseProcessorTests
    {
        #region Public Methods and Operators

        /// <summary>
        /// The approved capture sets all appropriate values.
        /// </summary>
        [TestMethod]
        public void ApprovedCaptureSetsAllAppropriateValues()
        {
            // Arrange
            const string AuthCode = "AuthCode";
            const string AvsResultCode = "AvsCode";
            const string RefNum = "1234";
            const string Result = "Result";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Approved, AuthCode = AuthCode, AvsResultCode = AvsResultCode, RefNum = RefNum, Result = Result, };
            var processPaymentResult = new CapturePaymentResult();

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessCapture(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(RefNum, processPaymentResult.CaptureTransactionId);
            Assert.AreEqual(Result, processPaymentResult.CaptureTransactionResult);
            Assert.AreEqual(PaymentStatus.Paid, processPaymentResult.NewPaymentStatus);
        }

        /// <summary>
        /// The approved partial refund sets all appropriate values.
        /// </summary>
        [TestMethod]
        public void ApprovedPartialRefundSetsAllAppropriateValues()
        {
            // Arrange
            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Approved, };
            var processPaymentResult = new RefundPaymentResult();

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessRefund(transactionResponse, processPaymentResult, true);

            // Assert
            Assert.AreEqual(PaymentStatus.PartiallyRefunded, processPaymentResult.NewPaymentStatus);
        }

        /// <summary>
        /// The approved refund sets all appropriate values.
        /// </summary>
        [TestMethod]
        public void ApprovedRefundSetsAllAppropriateValues()
        {
            // Arrange
            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Approved, };
            var processPaymentResult = new RefundPaymentResult();

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessRefund(transactionResponse, processPaymentResult, false);

            // Assert
            Assert.AreEqual(PaymentStatus.Refunded, processPaymentResult.NewPaymentStatus);
        }

        /// <summary>
        ///     The approved transaction sets all appropriate values.
        /// </summary>
        [TestMethod]
        public void ApprovedTransactionSetsAllAppropriateValues()
        {
            // Arrange
            const string AuthCode = "AuthCode";
            const string AvsResultCode = "AvsCode";
            const string RefNum = "1234";
            const string Result = "Result";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Approved, AuthCode = AuthCode, AvsResultCode = AvsResultCode, RefNum = RefNum, Result = Result, };
            var processPaymentResult = new ProcessPaymentResult();

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessTransaction(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(false, processPaymentResult.AllowStoringCreditCardNumber);
            Assert.AreEqual(AuthCode, processPaymentResult.AuthorizationTransactionCode);
            Assert.AreEqual(AvsResultCode, processPaymentResult.AvsResult);
            Assert.AreEqual(RefNum, processPaymentResult.CaptureTransactionId);
            Assert.AreEqual(Result, processPaymentResult.CaptureTransactionResult);
            Assert.AreEqual(PaymentStatus.Paid, processPaymentResult.NewPaymentStatus);
        }

        /// <summary>
        /// The declined capture does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void DeclinedCaptureDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Declined, Error = Error, };
            var processPaymentResult = new CapturePaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessCapture(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Capture was declined. {0}", Error), processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The declined refund does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void DeclinedRefundDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Declined, Error = Error, };
            var processPaymentResult = new RefundPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessRefund(transactionResponse, processPaymentResult, false);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Refund was declined. {0}", Error), processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The declined transaction does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void DeclinedTransactionDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Declined, Error = Error, };
            var processPaymentResult = new ProcessPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessTransaction(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(false, processPaymentResult.AllowStoringCreditCardNumber);
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Payment was declined. {0}", Error), processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The error capture does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void ErrorCaptureDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Error, Error = Error, };
            var processPaymentResult = new CapturePaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessCapture(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(Error, processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The error refund does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void ErrorRefundDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Error, Error = Error, };
            var processPaymentResult = new RefundPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessRefund(transactionResponse, processPaymentResult, false);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(Error, processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The error transaction does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void ErrorTransactionDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Error, Error = Error, };
            var processPaymentResult = new ProcessPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessTransaction(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(false, processPaymentResult.AllowStoringCreditCardNumber);
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(Error, processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The verification capture does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void VerificationCaptureDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Verification, Error = Error, };
            var processPaymentResult = new CapturePaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessCapture(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Capture needs verification. {0}", Error), processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The verification refund does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void VerificationRefundDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Verification, Error = Error, };
            var processPaymentResult = new RefundPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessRefund(transactionResponse, processPaymentResult, false);

            // Assert
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Refund needs verification. {0}", Error), processPaymentResult.Errors.First());
        }

        /// <summary>
        /// The verification transaction does not change payment status and sets error.
        /// </summary>
        [TestMethod]
        public void VerificationTransactionDoesNotChangePaymentStatusAndSetsError()
        {
            // Arrange
            const string Error = "Bad thing";

            var transactionResponse = new TransactionResponse { ResultCode = TransactionResultCode.Verification, Error = Error, };
            var processPaymentResult = new ProcessPaymentResult { NewPaymentStatus = PaymentStatus.Pending, };

            // Act
            processPaymentResult = TransactionResponseProcessor.ProcessTransaction(transactionResponse, processPaymentResult);

            // Assert
            Assert.AreEqual(false, processPaymentResult.AllowStoringCreditCardNumber);
            Assert.AreEqual(PaymentStatus.Pending, processPaymentResult.NewPaymentStatus);
            Assert.AreEqual(string.Format("Payment needs verification. {0}", Error), processPaymentResult.Errors.First());
        }

        #endregion
    }
}