﻿namespace Nop.Plugin.Payments.UsaEpay.Tests
{
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    using Moq;

    using Nop.Core.Domain.Orders;
    using Nop.Core.Domain.Payments;
    using Nop.Services.Configuration;
    using Nop.Services.Orders;
    using Nop.Services.Payments;

    /// <summary>
    /// The USA ePay payment processor tests.
    /// </summary>
    [TestClass]
    public class UsaEpayPaymentProcessorTests
    {
        /// <summary>
        /// The constructor initializes service url with settings.
        /// </summary>
        [TestMethod]
        public void ConstructorInitializesAsExpected()
        {
            // Arrange
            const string ServiceUrl = "http://test.com";
            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings { Url = ServiceUrl };
            var usaEpayService = new Mock<IUsaEpayService>();
            usaEpayService.SetupProperty(s => s.Url);

            // Act
            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object, 
                settingService.Object, 
                usaEpayPaymentSettings, 
                usaEpayService.Object);
            
            // Assert
            Assert.IsNotNull(processor);
            Assert.AreEqual(ServiceUrl, usaEpayService.Object.Url);
            Assert.IsTrue(processor.SupportCapture);
            Assert.IsTrue(processor.SupportPartiallyRefund);
            Assert.IsTrue(processor.SupportRefund);
            Assert.IsTrue(processor.SupportVoid);
            Assert.AreEqual(PaymentMethodType.Standard, processor.PaymentMethodType);
            Assert.AreEqual(RecurringPaymentType.NotSupported, processor.RecurringPaymentType);
        }

        /// <summary>
        /// The can repost process payment always returns false.
        /// </summary>
        [TestMethod]
        public void CanRePostProcessPaymentAlwaysReturnsFalse()
        {
            // Arrange
            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings();
            var usaEpayService = new Mock<IUsaEpayService>();
            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object,
                settingService.Object,
                usaEpayPaymentSettings,
                usaEpayService.Object);
            var order = new Order();

            // Act
            var result = processor.CanRePostProcessPayment(order);

            // Assert
            Assert.IsFalse(result);
        }

        /// <summary>
        /// The when capture is called the correct fields are passed to service and a result is returned.
        /// </summary>
        [TestMethod]
        public void WhenCaptureIsCalledTheCorrectFieldsArePassedToServiceAndAResultIsReturned()
        {
            // Arrange
            const string ExpectedTransactionId = "10";
            const decimal OrderTotal = 20.21M;

            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings();
            var usaEpayService = new Mock<IUsaEpayService>();
            usaEpayService.Setup(s => s.captureTransaction(It.IsAny<ueSecurityToken>(), It.IsAny<string>(), It.IsAny<double>())).Returns(
                (ueSecurityToken token, string transactionId, decimal total) =>
                    {
                        Assert.AreEqual(ExpectedTransactionId, transactionId);
                        Assert.AreEqual(OrderTotal, total);
                        return new TransactionResponse
                                   {
                                       ResultCode = TransactionResultCode.Approved
                                   };
                    });
            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object,
                settingService.Object,
                usaEpayPaymentSettings,
                usaEpayService.Object);
            var order = new Order { AuthorizationTransactionId = ExpectedTransactionId, OrderTotal = OrderTotal };

            var capturePaymentRequest = new CapturePaymentRequest
                                            {
                                                Order = order
                                            };

            // Act
            var result = processor.Capture(capturePaymentRequest);

            // Assert
            Assert.IsNotNull(result);
        }

        /// <summary>
        /// The when process payment is called the correct fields are passed to service and a result is returned.
        /// </summary>
        [TestMethod]
        public void WhenProcessPaymentIsCalledTheCorrectFieldsArePassedToServiceAndAResultIsReturned()
        {
            // Arrange
            const string ExpectedAccountHolder = "Jim Smith";
            const decimal ExpectedAmount = 20.21M;
            const string ExpectedInvoice = "INVOICE";
            const string ExpectedCavv = "CAVV";
            const string ExpectedCardType = "CardType";
            const string ExpectedCardNumber = "4111111111111111";
            const string ExpectedCardExpiration = "0101";

            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings();
            var usaEpayService = new Mock<IUsaEpayService>();
            usaEpayService.Setup(s => s.runTransaction(It.IsAny<ueSecurityToken>(), It.IsAny<TransactionRequestObject>())).Returns(
                (ueSecurityToken token, TransactionRequestObject transaction) =>
                {
                    Assert.AreEqual(ExpectedAccountHolder, transaction.AccountHolder);
                    Assert.AreEqual(ExpectedCardExpiration, transaction.CreditCardData.CardExpiration);
                    Assert.AreEqual(ExpectedCardNumber, transaction.CreditCardData.CardNumber);
                    Assert.AreEqual(ExpectedCardType, transaction.CreditCardData.CardType);
                    Assert.AreEqual(ExpectedCavv, transaction.CreditCardData.CAVV);
                    Assert.AreEqual(ExpectedAmount, transaction.Details.Amount);
                    Assert.AreEqual(true, transaction.Details.AmountSpecified);
                    Assert.AreEqual(ExpectedInvoice, transaction.Details.Invoice);
                    return new TransactionResponse
                    {
                        ResultCode = TransactionResultCode.Approved
                    };
                });
            
            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object,
                settingService.Object,
                usaEpayPaymentSettings,
                usaEpayService.Object);
            
            var processPaymentRequest = new ProcessPaymentRequest
                                            {
                                                CreditCardName = ExpectedAccountHolder,
                                                CreditCardExpireMonth = 1,
                                                CreditCardExpireYear = 1,
                                                CreditCardNumber = ExpectedCardNumber,
                                                CreditCardType = ExpectedCardType,
                                                CreditCardCvv2 = ExpectedCavv,
                                                OrderTotal = ExpectedAmount,
                                                PurchaseOrderNumber = ExpectedInvoice
                                            };

            // Act
            var result = processor.ProcessPayment(processPaymentRequest);

            // Assert
            Assert.IsNotNull(result);
        }

        /// <summary>
        /// The when refund is called the correct fields are passed to service and a result is returned.
        /// </summary>
        [TestMethod]
        public void WhenRefundIsCalledTheCorrectFieldsArePassedToServiceAndAResultIsReturned()
        {
            // Arrange
            const string ExpectedCaptureTransactionId = "101";
            const decimal ExpectedAmount = 20.21M;

            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings();
            var usaEpayService = new Mock<IUsaEpayService>();
            usaEpayService.Setup(s => s.refundTransaction(It.IsAny<ueSecurityToken>(), It.IsAny<string>(), It.IsAny<double>())).Returns(
                (ueSecurityToken token, string transactionId, double amount) =>
                {
                    Assert.AreEqual(ExpectedCaptureTransactionId, transactionId);
                    Assert.AreEqual(ExpectedAmount, amount);

                    return new TransactionResponse
                    {
                        ResultCode = TransactionResultCode.Approved
                    };
                });

            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object,
                settingService.Object,
                usaEpayPaymentSettings,
                usaEpayService.Object);

            var order = new Order { PaymentStatus = PaymentStatus.Paid, CaptureTransactionId = ExpectedCaptureTransactionId };

            var refundPaymentRequest = new RefundPaymentRequest { Order = order, AmountToRefund = ExpectedAmount };

            // Act
            var result = processor.Refund(refundPaymentRequest);

            // Assert
            Assert.IsNotNull(result);
        }

        /// <summary>
        /// The when void is called the correct fields are passed to service and a result is returned.
        /// </summary>
        [TestMethod]
        public void WhenVoidIsCalledTheCorrectFieldsArePassedToServiceAndAResultIsReturned()
        {
            // Arrange
            const string ExpectedAuthorizationTransactionId = "101";

            var orderTotalCalculationService = new Mock<IOrderTotalCalculationService>();
            var settingService = new Mock<ISettingService>();
            var usaEpayPaymentSettings = new UsaEpayPaymentSettings();
            var usaEpayService = new Mock<IUsaEpayService>();
            usaEpayService.Setup(s => s.voidTransaction(It.IsAny<ueSecurityToken>(), It.IsAny<string>())).Returns(
                (ueSecurityToken token, string transactionId) =>
                {
                    Assert.AreEqual(ExpectedAuthorizationTransactionId, transactionId);
                    return true;
                });

            var processor = new UsaEpayPaymentProcessor(
                orderTotalCalculationService.Object,
                settingService.Object,
                usaEpayPaymentSettings,
                usaEpayService.Object);

            var order = new Order { PaymentStatus = PaymentStatus.Authorized, AuthorizationTransactionId = ExpectedAuthorizationTransactionId };

            var voidPaymentRequest = new VoidPaymentRequest { Order = order, };

            // Act
            var result = processor.Void(voidPaymentRequest);

            // Assert
            Assert.IsNotNull(result);
        }
    }
}
